@@ -31,6 +31,7 @@
 #define PSUTIL_TV2DOUBLE(t)    ((t).tv_sec + (t).tv_usec / 1000000.0)
 #define PSUTIL_BT2MSEC(bt) (bt.sec * 1000 + (((uint64_t) 1000000000 * (uint32_t) \
         (bt.frac >> 32) ) >> 32 ) / 1000000)
+#define DECIKELVIN_2_CELCIUS(t) (t - 2731) / 10
 #ifndef _PATH_DEVNULL
 #define _PATH_DEVNULL "/dev/null"
 #endif
@@ -53,7 +54,7 @@ psutil_kinfo_proc(const pid_t pid, struct kinfo_proc *proc) {
 
     size = sizeof(struct kinfo_proc);
     if (sysctl((int *)mib, 4, proc, &size, NULL, 0) == -1) {
-        PyErr_SetFromErrno(PyExc_OSError);
+        PyErr_SetFromOSErrnoWithSyscall("sysctl(KERN_PROC_PID)");
         return -1;
     }
 
@@ -208,7 +209,7 @@ static char
     size = argmax;
     if (sysctl(mib, 4, procargs, &size, NULL, 0) == -1) {
         free(procargs);
-        PyErr_SetFromErrno(PyExc_OSError);
+        PyErr_SetFromOSErrnoWithSyscall("sysctl(KERN_PROC_ARGS)");
         return NULL;
     }
 
@@ -273,7 +274,6 @@ psutil_proc_exe(PyObject *self, PyObject *args) {
     int mib[4];
     int ret;
     size_t size;
-    const char *encoding_errs;
 
     if (! PyArg_ParseTuple(args, "l", &pid))
         return NULL;
@@ -287,10 +287,13 @@ psutil_proc_exe(PyObject *self, PyObject *args) {
     error = sysctl(mib, 4, pathname, &size, NULL, 0);
     if (error == -1) {
         // see: https://github.com/giampaolo/psutil/issues/907
-        if (errno == ENOENT)
+        if (errno == ENOENT) {
             return PyUnicode_DecodeFSDefault("");
-        else
-            return PyErr_SetFromErrno(PyExc_OSError);
+        }
+        else {
+            return \
+                PyErr_SetFromOSErrnoWithSyscall("sysctl(KERN_PROC_PATHNAME)");
+        }
     }
     if (size == 0 || strlen(pathname) == 0) {
         ret = psutil_pid_exists(pid);
@@ -350,7 +353,7 @@ psutil_proc_threads(PyObject *self, PyObject *args) {
     size = 0;
     error = sysctl(mib, 4, NULL, &size, NULL, 0);
     if (error == -1) {
-        PyErr_SetFromErrno(PyExc_OSError);
+        PyErr_SetFromOSErrnoWithSyscall("sysctl(KERN_PROC_INC_THREAD)");
         goto error;
     }
     if (size == 0) {
@@ -366,7 +369,7 @@ psutil_proc_threads(PyObject *self, PyObject *args) {
 
     error = sysctl(mib, 4, kip, &size, NULL, 0);
     if (error == -1) {
-        PyErr_SetFromErrno(PyExc_OSError);
+        PyErr_SetFromOSErrnoWithSyscall("sysctl(KERN_PROC_INC_THREAD)");
         goto error;
     }
     if (size == 0) {
@@ -447,26 +450,38 @@ psutil_virtual_mem(PyObject *self, PyObject *args) {
 #endif
     size_t buffers_size = sizeof(buffers);
 
-    if (sysctlbyname("hw.physmem", &total, &size, NULL, 0))
-        goto error;
-    if (sysctlbyname("vm.stats.vm.v_active_count", &active, &size, NULL, 0))
-        goto error;
-    if (sysctlbyname("vm.stats.vm.v_inactive_count",
-                     &inactive, &size, NULL, 0))
-        goto error;
-    if (sysctlbyname("vm.stats.vm.v_wire_count", &wired, &size, NULL, 0))
-        goto error;
+    if (sysctlbyname("hw.physmem", &total, &size, NULL, 0)) {
+        return PyErr_SetFromOSErrnoWithSyscall("sysctlbyname('hw.physmem')");
+    }
+    if (sysctlbyname("vm.stats.vm.v_active_count", &active, &size, NULL, 0)) {
+        return PyErr_SetFromOSErrnoWithSyscall(
+            "sysctlbyname('vm.stats.vm.v_active_count')");
+    }
+    if (sysctlbyname("vm.stats.vm.v_inactive_count", &inactive, &size, NULL, 0))
+    {
+        return PyErr_SetFromOSErrnoWithSyscall(
+            "sysctlbyname('vm.stats.vm.v_inactive_count')");
+    }
+    if (sysctlbyname("vm.stats.vm.v_wire_count", &wired, &size, NULL, 0)) {
+        return PyErr_SetFromOSErrnoWithSyscall(
+            "sysctlbyname('vm.stats.vm.v_wire_count')");
+    }
     // https://github.com/giampaolo/psutil/issues/997
-    if (sysctlbyname("vm.stats.vm.v_cache_count", &cached, &size, NULL, 0))
+    if (sysctlbyname("vm.stats.vm.v_cache_count", &cached, &size, NULL, 0)) {
         cached = 0;
-    if (sysctlbyname("vm.stats.vm.v_free_count", &free, &size, NULL, 0))
-        goto error;
-    if (sysctlbyname("vfs.bufspace", &buffers, &buffers_size, NULL, 0))
-        goto error;
+    }
+    if (sysctlbyname("vm.stats.vm.v_free_count", &free, &size, NULL, 0)) {
+        return PyErr_SetFromOSErrnoWithSyscall(
+            "sysctlbyname('vm.stats.vm.v_free_count')");
+    }
+    if (sysctlbyname("vfs.bufspace", &buffers, &buffers_size, NULL, 0)) {
+        return PyErr_SetFromOSErrnoWithSyscall("sysctlbyname('vfs.bufspace')");
+    }
 
     size = sizeof(vm);
-    if (sysctl(mib, 2, &vm, &size, NULL, 0) != 0)
-        goto error;
+    if (sysctl(mib, 2, &vm, &size, NULL, 0) != 0) {
+        return PyErr_SetFromOSErrnoWithSyscall("sysctl(CTL_VM | VM_METER)");
+    }
 
     return Py_BuildValue("KKKKKKKK",
         (unsigned long long) total,
@@ -478,9 +493,6 @@ psutil_virtual_mem(PyObject *self, PyObject *args) {
         (unsigned long long) buffers,
         (unsigned long long) (vm.t_vmshr + vm.t_rmshr) * pagesize  // shared
     );
-
-error:
-    return PyErr_SetFromErrno(PyExc_OSError);
 }
 
 
@@ -491,6 +503,7 @@ psutil_swap_mem(PyObject *self, PyObject *args) {
     struct kvm_swap kvmsw[1];
     unsigned int swapin, swapout, nodein, nodeout;
     size_t size = sizeof(unsigned int);
+    int pagesize;
 
     kd = kvm_open(NULL, _PATH_DEVNULL, NULL, O_RDONLY, "kvm_open failed");
     if (kd == NULL) {
@@ -507,16 +520,28 @@ psutil_swap_mem(PyObject *self, PyObject *args) {
 
     kvm_close(kd);
 
-    if (sysctlbyname("vm.stats.vm.v_swapin", &swapin, &size, NULL, 0) == -1)
-        goto error;
-    if (sysctlbyname("vm.stats.vm.v_swapout", &swapout, &size, NULL, 0) == -1)
-        goto error;
-    if (sysctlbyname("vm.stats.vm.v_vnodein", &nodein, &size, NULL, 0) == -1)
-        goto error;
-    if (sysctlbyname("vm.stats.vm.v_vnodeout", &nodeout, &size, NULL, 0) == -1)
-        goto error;
+    if (sysctlbyname("vm.stats.vm.v_swapin", &swapin, &size, NULL, 0) == -1) {
+        return PyErr_SetFromOSErrnoWithSyscall(
+            "sysctlbyname('vm.stats.vm.v_swapin)'");
+    }
+    if (sysctlbyname("vm.stats.vm.v_swapout", &swapout, &size, NULL, 0) == -1){
+        return PyErr_SetFromOSErrnoWithSyscall(
+            "sysctlbyname('vm.stats.vm.v_swapout)'");
+    }
+    if (sysctlbyname("vm.stats.vm.v_vnodein", &nodein, &size, NULL, 0) == -1) {
+        return PyErr_SetFromOSErrnoWithSyscall(
+            "sysctlbyname('vm.stats.vm.v_vnodein)'");
+    }
+    if (sysctlbyname("vm.stats.vm.v_vnodeout", &nodeout, &size, NULL, 0) == -1) {
+        return PyErr_SetFromOSErrnoWithSyscall(
+            "sysctlbyname('vm.stats.vm.v_vnodeout)'");
+    }
 
-    int pagesize = getpagesize();
+    pagesize = getpagesize();
+    if (pagesize <= 0) {
+        PyErr_SetString(PyExc_ValueError, "invalid getpagesize()");
+        return NULL;
+    }
 
     return Py_BuildValue(
         "(KKKII)",
@@ -527,9 +552,6 @@ psutil_swap_mem(PyObject *self, PyObject *args) {
         swapin + swapout,  // swap in
         nodein + nodeout  // swap out
     );
-
-error:
-    return PyErr_SetFromErrno(PyExc_OSError);
 }
 
 
@@ -540,7 +562,6 @@ psutil_proc_cwd(PyObject *self, PyObject *args) {
     struct kinfo_file *freep = NULL;
     struct kinfo_file *kif;
     struct kinfo_proc kipp;
-    const char *encoding_errs;
     PyObject *py_path = NULL;
 
     int i, cnt;
@@ -630,7 +651,8 @@ psutil_per_cpu_times(PyObject *self, PyObject *args) {
     size = sizeof(maxcpus);
     if (sysctlbyname("kern.smp.maxcpus", &maxcpus, &size, NULL, 0) < 0) {
         Py_DECREF(py_retlist);
-        return PyErr_SetFromErrno(PyExc_OSError);
+        return PyErr_SetFromOSErrnoWithSyscall(
+            "sysctlbyname('kern.smp.maxcpus')");
     }
     long cpu_time[maxcpus][CPUSTATES];
 
@@ -639,14 +661,14 @@ psutil_per_cpu_times(PyObject *self, PyObject *args) {
     mib[1] = HW_NCPU;
     len = sizeof(ncpu);
     if (sysctl(mib, 2, &ncpu, &len, NULL, 0) == -1) {
-        PyErr_SetFromErrno(PyExc_OSError);
+        PyErr_SetFromOSErrnoWithSyscall("sysctl(HW_NCPU)");
         goto error;
     }
 
     // per-cpu info
     size = sizeof(cpu_time);
     if (sysctlbyname("kern.cp_times", &cpu_time, &size, NULL, 0) == -1) {
-        PyErr_SetFromErrno(PyExc_OSError);
+        PyErr_SetFromOSErrnoWithSyscall("sysctlbyname('kern.smp.maxcpus')");
         goto error;
     }
 
@@ -961,16 +983,26 @@ psutil_cpu_stats(PyObject *self, PyObject *args) {
     unsigned int v_swtch;
     size_t size = sizeof(v_soft);
 
-    if (sysctlbyname("vm.stats.sys.v_soft", &v_soft, &size, NULL, 0))
-        goto error;
-    if (sysctlbyname("vm.stats.sys.v_intr", &v_intr, &size, NULL, 0))
-        goto error;
-    if (sysctlbyname("vm.stats.sys.v_syscall", &v_syscall, &size, NULL, 0))
-        goto error;
-    if (sysctlbyname("vm.stats.sys.v_trap", &v_trap, &size, NULL, 0))
-        goto error;
-    if (sysctlbyname("vm.stats.sys.v_swtch", &v_swtch, &size, NULL, 0))
-        goto error;
+    if (sysctlbyname("vm.stats.sys.v_soft", &v_soft, &size, NULL, 0)) {
+        return PyErr_SetFromOSErrnoWithSyscall(
+            "sysctlbyname('vm.stats.sys.v_soft')");
+    }
+    if (sysctlbyname("vm.stats.sys.v_intr", &v_intr, &size, NULL, 0)) {
+        return PyErr_SetFromOSErrnoWithSyscall(
+            "sysctlbyname('vm.stats.sys.v_intr')");
+    }
+    if (sysctlbyname("vm.stats.sys.v_syscall", &v_syscall, &size, NULL, 0)) {
+        return PyErr_SetFromOSErrnoWithSyscall(
+            "sysctlbyname('vm.stats.sys.v_syscall')");
+    }
+    if (sysctlbyname("vm.stats.sys.v_trap", &v_trap, &size, NULL, 0)) {
+        return PyErr_SetFromOSErrnoWithSyscall(
+            "sysctlbyname('vm.stats.sys.v_trap')");
+    }
+    if (sysctlbyname("vm.stats.sys.v_swtch", &v_swtch, &size, NULL, 0)) {
+        return PyErr_SetFromOSErrnoWithSyscall(
+            "sysctlbyname('vm.stats.sys.v_swtch')");
+    }
 
     return Py_BuildValue(
         "IIIII",
@@ -980,9 +1012,6 @@ psutil_cpu_stats(PyObject *self, PyObject *args) {
         v_syscall,  // syscalls
         v_trap  // traps
     );
-
-error:
-    return PyErr_SetFromErrno(PyExc_OSError);
 }
 
 
@@ -1012,3 +1041,75 @@ error:
         PyErr_SetFromErrno(PyExc_OSError);
     return NULL;
 }
+
+
+/*
+ * Return temperature information for a given CPU core number.
+ */
+PyObject *
+psutil_sensors_cpu_temperature(PyObject *self, PyObject *args) {
+    int current;
+    int tjmax;
+    int core;
+    char sensor[26];
+    size_t size = sizeof(current);
+
+    if (! PyArg_ParseTuple(args, "i", &core))
+        return NULL;
+    sprintf(sensor, "dev.cpu.%d.temperature", core);
+    if (sysctlbyname(sensor, &current, &size, NULL, 0))
+        goto error;
+    current = DECIKELVIN_2_CELCIUS(current);
+
+    // Return -273 in case of faliure.
+    sprintf(sensor, "dev.cpu.%d.coretemp.tjmax", core);
+    if (sysctlbyname(sensor, &tjmax, &size, NULL, 0))
+        tjmax = 0;
+    tjmax = DECIKELVIN_2_CELCIUS(tjmax);
+
+    return Py_BuildValue("ii", current, tjmax);
+
+error:
+    if (errno == ENOENT)
+        PyErr_SetString(PyExc_NotImplementedError, "no temperature sensors");
+    else
+        PyErr_SetFromErrno(PyExc_OSError);
+    return NULL;
+}
+
+
+/*
+ * Return frequency information of a given CPU.
+ * As of Dec 2018 only CPU 0 appears to be supported and all other
+ * cores match the frequency of CPU 0.
+ */
+PyObject *
+psutil_cpu_freq(PyObject *self, PyObject *args) {
+    int current;
+    int core;
+    char sensor[26];
+    char available_freq_levels[1000];
+    size_t size = sizeof(current);
+
+    if (! PyArg_ParseTuple(args, "i", &core))
+        return NULL;
+    // https://www.unix.com/man-page/FreeBSD/4/cpufreq/
+    sprintf(sensor, "dev.cpu.%d.freq", core);
+    if (sysctlbyname(sensor, &current, &size, NULL, 0))
+        goto error;
+
+    size = sizeof(available_freq_levels);
+    // https://www.unix.com/man-page/FreeBSD/4/cpufreq/
+    // In case of failure, an empty string is returned.
+    sprintf(sensor, "dev.cpu.%d.freq_levels", core);
+    sysctlbyname(sensor, &available_freq_levels, &size, NULL, 0);
+
+    return Py_BuildValue("is", current, available_freq_levels);
+
+error:
+    if (errno == ENOENT)
+        PyErr_SetString(PyExc_NotImplementedError, "unable to read frequency");
+    else
+        PyErr_SetFromErrno(PyExc_OSError);
+    return NULL;
+}
